from django.test import TestCase

# from django.urls import reverse
# from django.contrib.auth.models import User
# from rest_framework import status
# from rest_framework.test import APITestCase
# from model_bakery import baker

PASSWORD = "jafsdfah24agdsfghasdf"


class DummyTest(TestCase):
    def test_dummy(self):
        self.assertTrue(True)


# class LoginTests(APITestCase):
#     def test_login_username(self):
#         """"
#         create standard user ai_kit_auth, log it in
#         """
#         user = baker.make(User)
#         url = reverse("rest_login")
#         response = self.client.post(
#             url, {"email_or_username": user.username, "password": PASSWORD}
#         )
#         self.assertEqual(response.status_code, status.HTTP_201_CREATED)



from django.contrib.auth import authenticate
from django.conf import settings
from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class LoginSerializer(serializers.Serializer):
    email_or_username = serializers.CharField(required=True)
    password = serializers.CharField(style={"input_type": "password"})

    def validate(self, attrs):
        ident = attrs.get("email_or_username")
        password = attrs.get("password")

        # try to log in with email
        user = authenticate(self.context["request"], email=ident, password=password)
        if not user:
            # try again with username
            user = authenticate(
                self.context["request"], username=ident, password=password
            )
        if not user:
            # still no luck, must be bogus credentials
            raise ValidationError("invalid_credentials")

        attrs["user"] = user
        return attrs



from datetime import datetime

from django.conf import settings
from django.contrib.auth.models import User
from rest_framework import viewsets, status, generics
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated, AllowAny
from rest_framework_jwt.settings import api_settings

from . import models, serializers


# utils
def jwt_encode(user):
    payload = api_settings.JWT_PAYLOAD_HANDLER(user)
    return api_settings.JWT_ENCODE_HANDLER(payload)


class LoginView(generics.GenericAPIView):
    # uses standard user model
    #
    serializer_class = serializers.LoginSerializer
    permission_classes = (AllowAny,)

    def post(self, request, *args, **kwargs):
        self.request = request
        self.serializer = self.get_serializer(
            data=self.request.data, context={"request": request}
        )
        self.serializer.is_valid(raise_exception=True)

        self.user = self.serializer.validated_data["user"]
        self.token = jwt_encode(self.user)

        data = {"user": self.user, "token": self.token}
        response = Response(data, status=status.HTTP_200_OK)

        return response



from django.urls import re_path
from . import views

# Right there is not a lot happening here, just the reexport of the rest_auth
# urls. Later when we do the ai_kit_auth reset stuff it makes sense to not use
# the rest_auth endpoints in the target app directly because we can do the
# configuration here that is a little bit fiddly


urlpatterns = [
    re_path(r"^login/$", views.LoginView.as_view(), name="rest_login"),
]
